/**************************************************************
	PIC melody
	 for the PIC12F629/675 (New complier)
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	version 1.00
	Tools:
	MPLAB v8.80
	Hitech PIC C pro9.83
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20110901 22:00	1.00	---		Begin the program
20111218 01:00	1.01	---		add "Jingle Bells" of new song data 

---------------------------------------------------------------
port assignment
1	VDD					8	VSS
2	RA5	O LED			7	RA0	(PGD)
3	RA4	(not used)		6	RA1	(PGC)
4	RA3	I SWITCH(VPP)	5	RA2	O	PIEZO_OUT

VDD 2.2V to 5V
RA3 is MUST PULLUP 10K OHM
attention:
Do not RA3 push-switch when Pickit2/ICD2 is active.

*/

#include <htc.h>
#include "keydefine.h"	//yf[^ keydefine.c

#pragma	jis
#define MHz 000000
#define _XTAL_FREQ 4MHz

//Chip configuration
__CONFIG(CP_OFF & BOREN_OFF & MCLRE_OFF & PWRTE_OFF & WDTE_OFF & FOSC_INTRCIO);	//Idoling current 0.06uA
//__CONFIG(CP_OFF & BOREN_ON & MCLRE_OFF & PWRTE_ON & WDTE_OFF & FOSC_INTRCIO);	//Idoling current 110uA

//output port					//	  76543210
#define sound_output 	0x04	//GP2 00000100
#define led_output 		0x20 	//GP5 00100000
#define keyinput 		0x08 	//GP3 00001000
//output port (individual)
#define SoundPort	GP2
#define LEDPort		GP5



//Tempo trim (0 to 4)
#define tempo_multi 8;

//Other define
#define Command_mode	255

//Global settings
unsigned char ToneSet;
unsigned char TempoCount = 100;	//Default TEMPO

//Interrupt service routine
void interrupt isr(void){
	TMR0 = ToneSet;
	if (ToneSet != 0) {
		if (SoundPort) SoundPort = 0 ;
			else SoundPort = 1; //alternate port 
	}
	T0IF =0;
}

//tempo control
void wtime(unsigned char t0){
	unsigned int t1;
	while(t0--){
		t1 = TempoCount * 4;
		while ( --t1 ){
			__delay_us(10);
		}	
	}
}

void sleep_control(void){
	LEDPort = 0;	//LED light
	__delay_ms(100);
	LEDPort = 1;	//LED off
	GIE  = 0;//Int off
	T0IE = 0;//timer interrupt is off
	while ( (GPIO & keyinput)==0 );//wait if key is pushing
	TRISIO = 0b11111111;//GPIO is all input
	NOP();	//Stabilize time
	WPU  = 0b00110111;//ALL Pin Pull Up(then Low power)
	IOCB = keyinput;	// checking switch port
	GPIF  = 0;//GPIF flag clear
	GPIE  = 1;//Sleep Wakeup enable
	GIE  = 0;//Interrupt service is disable

	//vector non jump and execute next command
	SLEEP();//sleep!
	NOP();//Wake up

	// Wakeup when switch is LOW 
	GPIE  = 0;//Sleep Wakeup disable
	GPIF  = 0;//GPIF flag clear
	T0IE = 1;//timer on
	GPIO = GPIO;	// Bug avoid
	TRISIO = ~(sound_output|led_output);
	GIE	 = 1; //Interrupt is All enable	
	GPIO &= ~led_output;//LED flash
	while ( (GPIO & keyinput)==0 );//wait if key is pushing
}
	
void delay_10ms(unsigned char temp){	//10ms multiple x times
	while(temp--){
		__delay_ms(10);
	}	
}
	
		
void main(void){
	unsigned int n=0;
	unsigned char tmp,tmp2;
	CMCON  = 7;		//All ports are digital I/O.
	#if _12F675
	ANSEL  = 0b00000000; //when device is 12F675
	#endif

	GPIO  = 0b00000000;
	TRISIO  = ~(sound_output|led_output);//LED and piezo (GP2,GP5)
	TMR0 =0x00 ;
	GIE  = 1;
	PEIE = 1;
	ToneSet =0x00;
	OPTION_REG = 0b00000010; // prescale 8 Pullup enable
	T0IE = 1;
	sleep_control();//sleep
	while(1){	
		//command detect
		tmp=songdata[n];
		tmp2=songdata[n+1];
		if (tmp == Command_mode ) {	//Command?
			if (tmp2 == END){	//Tempo or End of data?
				n=0;
				sleep_control();//sleep
			}
			else{
				TempoCount = tmp2;	//New tempo change
				n +=2;
			}
		}
		else{
			if ((GPIO & keyinput)==0) sleep_control();// resume
			ToneSet=tmp;
			wtime(tmp2);
			GPIO ^= led_output;//LED flash
			n +=2;		
		}
	}
}
